@@ -1,10 +1,10 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "ob-llm" "0.1.0"
+(define-package "ob-llm" "20250630.303"
   "Use `llm' as an Org Babel language."
   '((emacs "27.1"))
   :url "https://github.com/sunflowerseastar/ob-llm"
-  :commit "35d14e29652d9532c03454524fff6f8baed94c45"
-  :revdesc "35d14e29652d"
+  :commit "f667dbce7fb30c8acac06a03e60447663616da12"
+  :revdesc "f667dbce7fb3"
   :keywords '("llm" "org-mode" "tools" "convenience" "org" "babel")
   :authors '(("Grant Surlyn" . "grant@sunflowerseastar.com"))
   :maintainers '(("Grant Surlyn" . "grant@sunflowerseastar.com")))
