@@ -4,8 +4,8 @@
 
 ;; Author: Grant Surlyn <grant@sunflowerseastar.com>
 ;; URL: https://github.com/sunflowerseastar/ob-llm
-;; Package-Version: 0.1.0
-;; Package-Revision: 35d14e29652d
+;; Package-Version: 20250630.303
+;; Package-Revision: f667dbce7fb3
 ;; Package-Requires: ((emacs "27.1"))
 ;; Keywords: llm org-mode tools convenience org babel
 ;; SPDX-License-Identifier: GPL-3.0-or-later
@@ -124,6 +124,34 @@ or flag with value. For example:
   :type '(repeat string)
   :group 'ob-llm)
 
+(defcustom ob-llm-post-process-auto-shift-p t
+  "Whether to adjust converted Org's heading levels.
+
+Here is an example with shifting:
+
+* heading 1
+** heading 2
+#+begin_src llm
+... prompt ...
+#+end_src
+
+#+RESULTS:
+*** response starts at heading 3
+
+
+And without:
+
+* heading 1
+** heading 2
+#+begin_src llm
+... prompt ...
+#+end_src
+
+#+RESULTS:
+* response starts back at heading 1"
+  :type 'boolean
+  :group 'ob-llm)
+
 (defcustom ob-llm-models nil
   "List of available LLM models.
 
@@ -140,6 +168,19 @@ know will be available to `llm'."
 ;;; General helpers
 ;;  ---------------------------------------------------------------------------
 
+(defun ob-llm--shell-quote-a-string-but-keep-spaces (string-with-spaces)
+  "Shell quote STRING-WITH-SPACES but don't escape spaces.
+
+This exists so a user can pass, ex. `:at myfile.org text/plain'
+and it can be converted to an llm-flag of `--at myfile.org
+text/plain' instead of `:at myfile.org\ text/plain'. The latter
+one has an escaped space, which means that `llm' will look for a
+file called `myfile.org\ text/plain'. The former will pass 2
+arguments to `--at', the first being the filename and the second
+being the content type."
+  (mapconcat (lambda (x) (shell-quote-argument x))
+             (split-string string-with-spaces " ") " "))
+
 (defun ob-llm--process-header-args (params)
   "Process header arguments in PARAMS and return categorized results.
 Returns a plist with keys:
@@ -160,7 +201,7 @@ Returns a plist with keys:
           (push param org-code-block-header-args))
 
          ;; ...special header arguments used as "params" for ob-llm application logic...
-         ((memq key '(:database :no-conversion))
+         ((memq key '(:user-path :no-conversion :no-shift))
           (push param custom-params))
 
          ;; ...and then all other header arguments are going to be passed to the
@@ -194,18 +235,22 @@ cleaned up."
      (let* ((key (symbol-name (car llm-param)))
             (flag-name (substring key 1)) ; remove leading colon
             (flag-value (cdr llm-param))
-            (single-letter-flag-name-p (= (length flag-name) 1)))
+            (flag-hyphens (if (= (length flag-name) 1) "-" "--")))
        (cond
         ;; boolean flag, ex. `:no-log' -> `--no-log '
         ((null flag-value)
-         (if single-letter-flag-name-p
-             (format "-%s " flag-name)  ; note the single hyphen
-           (format "--%s " flag-name))) ; note the double hyphen
-        ;; flag with a value, ex. `:m 4o' -> `-m 4o '
-        (t
-         (if single-letter-flag-name-p
-             (format "-%s %s " flag-name (shell-quote-argument (format "%s" flag-value)))
-           (format "--%s %s " flag-name (shell-quote-argument (format "%s" flag-value))))))))
+         (concat flag-hyphens (format "%s " flag-name)))
+        ;; --attachment-type (or --at) takes two arguments
+        ((or (equal flag-name "attachment-type") (equal flag-name "at"))
+         (concat flag-hyphens (format "%s %s " flag-name
+                                      (ob-llm--shell-quote-a-string-but-keep-spaces flag-value))))
+        ;; the database path needs to be expanded
+        ((or (equal flag-name "database") (equal flag-name "d"))
+         (concat flag-hyphens (format "%s %s " flag-name
+                                      (expand-file-name flag-value))))
+        ;; flag with a single value, ex. `:m 4o' -> `-m 4o '
+        (t (concat flag-hyphens (format "%s %s " flag-name
+                                        (shell-quote-argument (format "%s" flag-value))))))))
    llm-params ""))
 
 (defun ob-llm--construct-llm-shell-command (body raw-params)
@@ -213,9 +258,9 @@ cleaned up."
   (let* ((processed-params (ob-llm--process-header-args raw-params))
          (llm-params (plist-get processed-params :llm-params))
          (llm-flags (ob-llm--llm-params->llm-flags llm-params))
-         (logs-database-path (alist-get :database raw-params)))
-    (format (concat (when logs-database-path
-                      (format "LLM_USER_PATH=%s " logs-database-path))
+         (custom-llm-user-path (alist-get :user-path raw-params)))
+    (format (concat (when custom-llm-user-path
+                      (format "LLM_USER_PATH=%s " custom-llm-user-path))
                     "llm %s %s")
             (shell-quote-argument body)
             llm-flags)))
@@ -260,40 +305,82 @@ cleaned up."
           final-output))
     final-output))
 
-(defun ob-llm--convert-markdown-response-to-org-mode (text)
-  "Convert markdown TEXT to Org with Pandoc, or return TEXT if conversion fails."
-  (if (executable-find "pandoc")
-      (with-temp-buffer
-        (insert text)
-        (let ((pandoc-args (append '("--from" "markdown+hard_line_breaks"
-                                     "--to" "org"
-                                     "--sandbox=true"
-                                     "--wrap=none")
-                                   ob-llm-pandoc-additional-org-mode-conversion-flags)))
-          (if (zerop (apply #'call-process-region
-                            (point-min) (point-max)
-                            "pandoc" t t nil
-                            pandoc-args))
-              (ob-llm--remove-trailing-backslashes (buffer-string))
-            text)))
-    text))
+(defun ob-llm--convert-markdown-response-to-org-mode (text &optional heading-offset)
+  "Convert markdown TEXT to Org and optionally shift headings.
 
-(defun ob-llm--post-process-result (final-output schema-p no-conversion-p)
+Step 1 is to run the markdown through `pandoc'.
+
+Step 2 is to take the HEADING-OFFSET, and shift all the Org
+syntax headings by that many levels by adding asterisks with
+`sed'.
+
+Let the user know if there are errors, and send back the text at
+that point."
+  (unless (executable-find "pandoc")
+    (message "ob-llm: pandoc not found – leaving markdown unconverted")
+    (cl-return-from ob-llm--convert-markdown-response-to-org-mode text))
+  (with-temp-buffer
+    (insert text)
+    (let* ((pandoc-args
+            (append '("--from" "markdown+hard_line_breaks-simple_tables" "--to" "org"
+                      "--sandbox=true" "--wrap=none")
+                    ob-llm-pandoc-additional-org-mode-conversion-flags))
+           (pandoc-status
+            ;; step 1 - convert markdown to org with pandoc
+            (apply #'call-process-region
+                   (point-min) (point-max)
+                   "pandoc" t t nil pandoc-args)))
+      (if (/= pandoc-status 0)
+          (progn
+            (message "ob-llm: pandoc exited non-zero when trying to convert markdown to Org syntax: %s" pandoc-status)
+            text)
+        ;; step 2 - optionally adjust the org heading levels
+        (let ((org-text (ob-llm--remove-trailing-backslashes (buffer-string))))
+          (cond
+           ;; 2a - there's no heading adjustment to make
+           ((or (not heading-offset) (<= heading-offset 0)) org-text)
+           ;; 2b - sed is not available
+           ((not (executable-find "sed"))
+            (message "ob-llm: sed not found – Org headings unshifted")
+            org-text)
+           ;; 2c - go for it, adjust the heading levels
+           (t (with-temp-buffer
+                (insert org-text)
+                (let* ((extra-asterisks (make-string heading-offset ?*))
+                       (script (format "s/^\\*\\{1,\\}/&%s/" extra-asterisks))
+                       (sed-status (call-process-region
+                                    (point-min) (point-max)
+                                    "sed" t t nil "-e" script)))
+                  (if (zerop sed-status)
+                      (buffer-string)
+                    (message "ob-llm: sed exited non-zero when trying to shift Org headings: %s"
+                             sed-status)
+                    (buffer-string)))))))))))
+
+(defun ob-llm--post-process-result (final-output schema-p no-conversion-p no-shift-p &optional heading-offset)
   "Post-process FINAL-OUTPUT.
 
-SCHEMA-P and NO-CONVERSION-P are used to determine how to
-process."
+If NO-CONVERSION-P (or if `ob-llm-post-process-auto-convert-p' is
+nil), skip any conversion. If SCHEMA-P, then prettify the JSON
+response.
+
+The HEADING-OFFSET says what level of nested headings the source
+Org buffer is at. This is thread through to the conversion
+function, which will use `sed` to adjust the headings all to a
+deeper level if necessary so that the response will neatly nest
+into the Org buffer. The HEADING-OFFSET is set to 0 (as in, no
+shifting) if `ob-llm-post-process-auto-shift-p' is nil or if
+NO-SHIFT-P was passed."
   (cond
-   ;; 1 - do not perform any post-process conversion
-   ((or no-conversion-p (not ob-llm-post-process-auto-convert-p)) final-output)
-
-   ;; 2 - it's a schema, so prettify the JSON response
+   ((or no-conversion-p (not ob-llm-post-process-auto-convert-p))
+    final-output)
    (schema-p
-    (let ((final-output-after-pp-json (ob-llm--prettify-json-response final-output)))
-      (format "#+begin_src json\n%s\n#+end_src" final-output-after-pp-json)))
-
-   ;; 3 - otherwise, convert the markdown response to Org mode syntax
-   (t (ob-llm--convert-markdown-response-to-org-mode final-output))))
+    (let ((pretty (ob-llm--prettify-json-response final-output)))
+      (format "#+begin_src json\n%s\n#+end_src" pretty)))
+   (t
+    (let* ((remove-heading-offset-p (or no-shift-p (not ob-llm-post-process-auto-shift-p)))
+           (adjusted-heading-offset (if remove-heading-offset-p 0 heading-offset)))
+      (ob-llm--convert-markdown-response-to-org-mode final-output adjusted-heading-offset)))))
 
 ;;  ---------------------------------------------------------------------------
 ;;; Streams and processes
@@ -454,8 +541,13 @@ RESULT-MARKER define where to stream in src Org buffer."
                                  (assq :schema-multi llm-params)))
                    (custom-params (plist-get processed-params :custom-params))
                    (no-conversion-p (assq :no-conversion custom-params))
+                   (no-shift-p (assq :no-shift custom-params))
+                   (heading-offset
+                    (org-with-point-at src-position
+                      (or (org-current-level) 0)))
                    (processed-final-output
-                    (ob-llm--post-process-result final-output schema-p no-conversion-p)))
+                    (ob-llm--post-process-result
+                     final-output schema-p no-conversion-p no-shift-p heading-offset)))
               (ob-llm--insert-output processed-final-output src-buffer src-position)))))
 
       ;; remove from active processes list
@@ -602,13 +694,13 @@ Each candidate is a cons cell where the car is a formatted string
 that displays conversation_name, prompt, conversation_model, and
 id in columns, and the cdr is the corresponding id."
   (cl-loop for entry across json-data
-           for conv = (alist-get 'conversation_name entry)
+           for name = (alist-get 'conversation_name entry)
            for prompt = (alist-get 'prompt entry)
            for model = (alist-get 'conversation_model entry)
            for id = (alist-get 'conversation_id entry)
            collect
            (cons (format "%-40s %-60s %-20s %s"
-                         (or conv "")
+                         (or name "")
                          (or prompt "")
                          (or model "")
                          (or id ""))
